@@ -11,11 +11,24 @@
 #include <rapidjson/document.h>
 #include <rapidjson/reader.h>
 
+#include <llvm/ADT/STLExtras.h>
+
 #include <algorithm>
 #include <stdexcept>
 
 using namespace clang;
 
+#if LLVM_VERSION_MAJOR < 15 // llvmorg-15-init-6118-gb39f43775796
+namespace llvm {
+template <typename T, typename E> constexpr bool is_contained(std::initializer_list<T> set, const E &e) {
+  for (const T &v : set)
+    if (v == e)
+      return true;
+  return false;
+}
+} // namespace llvm
+#endif
+
 MAKE_HASHABLE(ccls::SymbolIdx, t.usr, t.kind);
 
 namespace ccls {
@@ -38,8 +51,7 @@ REFLECT_STRUCT(CompletionParam, textDocument, position, context);
 // formatting
 REFLECT_STRUCT(FormattingOptions, tabSize, insertSpaces);
 REFLECT_STRUCT(DocumentFormattingParam, textDocument, options);
-REFLECT_STRUCT(DocumentOnTypeFormattingParam, textDocument, position, ch,
-               options);
+REFLECT_STRUCT(DocumentOnTypeFormattingParam, textDocument, position, ch, options);
 REFLECT_STRUCT(DocumentRangeFormattingParam, textDocument, range, options);
 
 // workspace
@@ -51,6 +63,10 @@ REFLECT_STRUCT(DidChangeWorkspaceFoldersParam, event);
 REFLECT_STRUCT(WorkspaceSymbolParam, query, folders);
 
 namespace {
+struct Occur {
+  lsRange range;
+  Role role;
+};
 struct CclsSemanticHighlightSymbol {
   int id = 0;
   SymbolKind parentKind;
@@ -58,16 +74,15 @@ struct CclsSemanticHighlightSymbol {
   uint8_t storage;
   std::vector<std::pair<int, int>> ranges;
 
-  // `lsRanges` is used to compute `ranges`.
-  std::vector<lsRange> lsRanges;
+  // `lsOccur` is used to compute `ranges`.
+  std::vector<Occur> lsOccurs;
 };
 
 struct CclsSemanticHighlight {
   DocumentUri uri;
   std::vector<CclsSemanticHighlightSymbol> symbols;
 };
-REFLECT_STRUCT(CclsSemanticHighlightSymbol, id, parentKind, kind, storage,
-               ranges, lsRanges);
+REFLECT_STRUCT(CclsSemanticHighlightSymbol, id, parentKind, kind, storage, ranges);
 REFLECT_STRUCT(CclsSemanticHighlight, uri, symbols);
 
 struct CclsSetSkippedRanges {
@@ -76,10 +91,16 @@ struct CclsSetSkippedRanges {
 };
 REFLECT_STRUCT(CclsSetSkippedRanges, uri, skippedRanges);
 
+struct SemanticTokensPartialResult {
+  std::vector<int> data;
+};
+REFLECT_STRUCT(SemanticTokensPartialResult, data);
+
 struct ScanLineEvent {
   Position pos;
   Position end_pos; // Second key when there is a tie for insertion events.
   int id;
+  Role role;
   CclsSemanticHighlightSymbol *symbol;
   bool operator<(const ScanLineEvent &o) const {
     // See the comments below when insertion/deletion events are inserted.
@@ -110,21 +131,15 @@ void ReplyOnce::replyLocationLink(std::vector<LocationLink> &result) {
   if (g_config->client.linkSupport) {
     (*this)(result);
   } else {
-    (*this)(std::vector<Location>(std::make_move_iterator(result.begin()),
-                                  std::make_move_iterator(result.end())));
+    (*this)(std::vector<Location>(std::make_move_iterator(result.begin()), std::make_move_iterator(result.end())));
   }
 }
 
-void MessageHandler::bind(const char *method,
-                          void (MessageHandler::*handler)(JsonReader &)) {
-  method2notification[method] = [this, handler](JsonReader &reader) {
-    (this->*handler)(reader);
-  };
+void MessageHandler::bind(const char *method, void (MessageHandler::*handler)(JsonReader &)) {
+  method2notification[method] = [this, handler](JsonReader &reader) { (this->*handler)(reader); };
 }
 
-template <typename Param>
-void MessageHandler::bind(const char *method,
-                          void (MessageHandler::*handler)(Param &)) {
+template <typename Param> void MessageHandler::bind(const char *method, void (MessageHandler::*handler)(Param &)) {
   method2notification[method] = [this, handler](JsonReader &reader) {
     Param param{};
     reflect(reader, param);
@@ -132,21 +147,13 @@ void MessageHandler::bind(const char *method,
   };
 }
 
-void MessageHandler::bind(const char *method,
-                          void (MessageHandler::*handler)(JsonReader &,
-                                                          ReplyOnce &)) {
-  method2request[method] = [this, handler](JsonReader &reader,
-                                           ReplyOnce &reply) {
-    (this->*handler)(reader, reply);
-  };
+void MessageHandler::bind(const char *method, void (MessageHandler::*handler)(JsonReader &, ReplyOnce &)) {
+  method2request[method] = [this, handler](JsonReader &reader, ReplyOnce &reply) { (this->*handler)(reader, reply); };
 }
 
 template <typename Param>
-void MessageHandler::bind(const char *method,
-                          void (MessageHandler::*handler)(Param &,
-                                                          ReplyOnce &)) {
-  method2request[method] = [this, handler](JsonReader &reader,
-                                           ReplyOnce &reply) {
+void MessageHandler::bind(const char *method, void (MessageHandler::*handler)(Param &, ReplyOnce &)) {
+  method2request[method] = [this, handler](JsonReader &reader, ReplyOnce &reply) {
     Param param{};
     reflect(reader, param);
     (this->*handler)(param, reply);
@@ -190,7 +197,10 @@ MessageHandler::MessageHandler() {
   bind("textDocument/rangeFormatting", &MessageHandler::textDocument_rangeFormatting);
   bind("textDocument/references", &MessageHandler::textDocument_references);
   bind("textDocument/rename", &MessageHandler::textDocument_rename);
+  bind("textDocument/semanticTokens/full", &MessageHandler::textDocument_semanticTokensFull);
+  bind("textDocument/semanticTokens/range", &MessageHandler::textDocument_semanticTokensRange);
   bind("textDocument/signatureHelp", &MessageHandler::textDocument_signatureHelp);
+  bind("textDocument/switchSourceHeader", &MessageHandler::textDocument_switchSourceHeader);
   bind("textDocument/typeDefinition", &MessageHandler::textDocument_typeDefinition);
   bind("workspace/didChangeConfiguration", &MessageHandler::workspace_didChangeConfiguration);
   bind("workspace/didChangeWatchedFiles", &MessageHandler::workspace_didChangeWatchedFiles);
@@ -213,13 +223,11 @@ void MessageHandler::run(InMessage &msg) {
         it->second(reader, reply);
       } catch (std::invalid_argument &ex) {
         reply.error(ErrorCode::InvalidParams,
-                    "invalid params of " + msg.method + ": expected " +
-                        ex.what() + " for " + reader.getPath());
+                    "invalid params of " + msg.method + ": expected " + ex.what() + " for " + reader.getPath());
       } catch (NotIndexed &) {
         throw;
       } catch (...) {
-        reply.error(ErrorCode::InternalError,
-                    "failed to process " + msg.method);
+        reply.error(ErrorCode::InternalError, "failed to process " + msg.method);
       }
     } else {
       reply.error(ErrorCode::MethodNotFound, "unknown request " + msg.method);
@@ -230,8 +238,7 @@ void MessageHandler::run(InMessage &msg) {
       try {
         it->second(reader);
       } catch (...) {
-        ShowMessageParam param{MessageType::Error,
-                               std::string("failed to process ") + msg.method};
+        ShowMessageParam param{MessageType::Error, std::string("failed to process ") + msg.method};
         pipeline::notify(window_showMessage, param);
       }
   }
@@ -254,9 +261,8 @@ QueryFile *MessageHandler::findFile(const std::string &path, int *out_file_id) {
   return ret;
 }
 
-std::pair<QueryFile *, WorkingFile *>
-MessageHandler::findOrFail(const std::string &path, ReplyOnce &reply,
-                           int *out_file_id, bool allow_unopened) {
+std::pair<QueryFile *, WorkingFile *> MessageHandler::findOrFail(const std::string &path, ReplyOnce &reply,
+                                                                 int *out_file_id, bool allow_unopened) {
   WorkingFile *wf = wfiles->getFile(path);
   if (!wf && !allow_unopened) {
     reply.notOpened(path);
@@ -281,16 +287,15 @@ void emitSkippedRanges(WorkingFile *wfile, QueryFile &file) {
   pipeline::notify("$ccls/publishSkippedRanges", params);
 }
 
-void emitSemanticHighlight(DB *db, WorkingFile *wfile, QueryFile &file) {
-  static GroupMatch match(g_config->highlight.whitelist,
-                          g_config->highlight.blacklist);
+static std::unordered_map<SymbolIdx, CclsSemanticHighlightSymbol> computeSemanticTokens(DB *db, WorkingFile *wfile,
+                                                                                        QueryFile &file) {
+  static GroupMatch match(g_config->highlight.whitelist, g_config->highlight.blacklist);
   assert(file.def);
-  if (wfile->buffer_content.size() > g_config->highlight.largeFileSize ||
-      !match.matches(file.def->path))
-    return;
-
   // Group symbols together.
   std::unordered_map<SymbolIdx, CclsSemanticHighlightSymbol> grouped_symbols;
+  if (!match.matches(file.def->path))
+    return grouped_symbols;
+
   for (auto [sym, refcnt] : file.symbol2refcnt) {
     if (refcnt <= 0)
       continue;
@@ -321,8 +326,7 @@ void emitSemanticHighlight(DB *db, WorkingFile *wfile, QueryFile &file) {
       // If not, do not publish the semantic highlight.
       // E.g. copy-initialization of constructors should not be highlighted
       // but we still want to keep the range for jumping to definition.
-      std::string_view concise_name =
-          detailed_name.substr(0, detailed_name.find('<'));
+      std::string_view concise_name = detailed_name.substr(0, detailed_name.find('<'));
       uint16_t start_line = sym.range.start.line;
       int16_t start_col = sym.range.start.column;
       if (start_line >= wfile->index_lines.size())
@@ -369,14 +373,14 @@ void emitSemanticHighlight(DB *db, WorkingFile *wfile, QueryFile &file) {
     if (std::optional<lsRange> loc = getLsRange(wfile, sym.range)) {
       auto it = grouped_symbols.find(sym);
       if (it != grouped_symbols.end()) {
-        it->second.lsRanges.push_back(*loc);
+        it->second.lsOccurs.push_back({*loc, sym.role});
       } else {
         CclsSemanticHighlightSymbol symbol;
         symbol.id = idx;
         symbol.parentKind = parent_kind;
         symbol.kind = kind;
         symbol.storage = storage;
-        symbol.lsRanges.push_back(*loc);
+        symbol.lsOccurs.push_back({*loc, sym.role});
         grouped_symbols[sym] = symbol;
       }
     }
@@ -387,17 +391,17 @@ void emitSemanticHighlight(DB *db, WorkingFile *wfile, QueryFile &file) {
   int id = 0;
   for (auto &entry : grouped_symbols) {
     CclsSemanticHighlightSymbol &symbol = entry.second;
-    for (auto &loc : symbol.lsRanges) {
+    for (auto &occur : symbol.lsOccurs) {
       // For ranges sharing the same start point, the one with leftmost end
       // point comes first.
-      events.push_back({loc.start, loc.end, id, &symbol});
+      events.push_back({occur.range.start, occur.range.end, id, occur.role, &symbol});
       // For ranges sharing the same end point, their relative order does not
-      // matter, therefore we arbitrarily assign loc.end to them. We use
+      // matter, therefore we arbitrarily assign occur.range.end to them. We use
       // negative id to indicate a deletion event.
-      events.push_back({loc.end, loc.end, ~id, &symbol});
+      events.push_back({occur.range.end, occur.range.end, ~id, occur.role, &symbol});
       id++;
     }
-    symbol.lsRanges.clear();
+    symbol.lsOccurs.clear();
   }
   std::sort(events.begin(), events.end());
 
@@ -413,26 +417,33 @@ void emitSemanticHighlight(DB *db, WorkingFile *wfile, QueryFile &file) {
     // Attribute range [events[i-1].pos, events[i].pos) to events[top-1].symbol
     // .
     if (top && !(events[i - 1].pos == events[i].pos))
-      events[top - 1].symbol->lsRanges.push_back(
-          {events[i - 1].pos, events[i].pos});
+      events[top - 1].symbol->lsOccurs.push_back({{events[i - 1].pos, events[i].pos}, events[i].role});
     if (events[i].id >= 0)
       events[top++] = events[i];
     else
       deleted[~events[i].id] = 1;
   }
+  return grouped_symbols;
+}
+
+void emitSemanticHighlight(DB *db, WorkingFile *wfile, QueryFile &file) {
+  // Disable $ccls/publishSemanticHighlight if semantic tokens support is
+  // enabled or the file is too large.
+  if (g_config->client.semanticTokensRefresh || wfile->buffer_content.size() > g_config->highlight.largeFileSize)
+    return;
+  auto grouped_symbols = computeSemanticTokens(db, wfile, file);
 
   CclsSemanticHighlight params;
   params.uri = DocumentUri::fromPath(wfile->filename);
   // Transform lsRange into pair<int, int> (offset pairs)
-  if (!g_config->highlight.lsRanges) {
-    std::vector<std::pair<lsRange, CclsSemanticHighlightSymbol *>> scratch;
+  {
+    std::vector<std::pair<Occur, CclsSemanticHighlightSymbol *>> scratch;
     for (auto &entry : grouped_symbols) {
-      for (auto &range : entry.second.lsRanges)
-        scratch.emplace_back(range, &entry.second);
-      entry.second.lsRanges.clear();
+      for (auto &occur : entry.second.lsOccurs)
+        scratch.push_back({occur, &entry.second});
+      entry.second.lsOccurs.clear();
     }
-    std::sort(scratch.begin(), scratch.end(),
-              [](auto &l, auto &r) { return l.first.start < r.first.start; });
+    std::sort(scratch.begin(), scratch.end(), [](auto &l, auto &r) { return l.first.range < r.first.range; });
     const auto &buf = wfile->buffer_content;
     int l = 0, c = 0, i = 0, p = 0;
     auto mov = [&](int line, int col) {
@@ -449,13 +460,12 @@ void emitSemanticHighlight(DB *db, WorkingFile *wfile, QueryFile &file) {
       for (; c < col && i < buf.size() && buf[i] != '\n'; c++)
         if (p++, uint8_t(buf[i++]) >= 128)
           // Skip 0b10xxxxxx
-          while (i < buf.size() && uint8_t(buf[i]) >= 128 &&
-                 uint8_t(buf[i]) < 192)
+          while (i < buf.size() && uint8_t(buf[i]) >= 128 && uint8_t(buf[i]) < 192)
             i++;
       return c < col;
     };
     for (auto &entry : scratch) {
-      lsRange &r = entry.first;
+      lsRange &r = entry.first.range;
       if (mov(r.start.line, r.start.character))
         continue;
       int beg = p;
@@ -466,8 +476,84 @@ void emitSemanticHighlight(DB *db, WorkingFile *wfile, QueryFile &file) {
   }
 
   for (auto &entry : grouped_symbols)
-    if (entry.second.ranges.size() || entry.second.lsRanges.size())
+    if (entry.second.ranges.size() || entry.second.lsOccurs.size())
       params.symbols.push_back(std::move(entry.second));
   pipeline::notify("$ccls/publishSemanticHighlight", params);
 }
+
+void MessageHandler::textDocument_semanticTokensFull(TextDocumentParam &param, ReplyOnce &reply) {
+  SemanticTokensRangeParams parameters{param.textDocument, lsRange{{0, 0}, {UINT16_MAX, INT16_MAX}}};
+  textDocument_semanticTokensRange(parameters, reply);
+}
+
+void MessageHandler::textDocument_semanticTokensRange(SemanticTokensRangeParams &param, ReplyOnce &reply) {
+  int file_id;
+  auto [file, wf] = findOrFail(param.textDocument.uri.getPath(), reply, &file_id);
+  if (!wf)
+    return;
+
+  auto grouped_symbols = computeSemanticTokens(db, wf, *file);
+  std::vector<std::pair<Occur, CclsSemanticHighlightSymbol *>> scratch;
+  for (auto &entry : grouped_symbols) {
+    for (auto &occur : entry.second.lsOccurs)
+      scratch.emplace_back(occur, &entry.second);
+    entry.second.lsOccurs.clear();
+  }
+  std::sort(scratch.begin(), scratch.end(), [](auto &l, auto &r) { return l.first.range < r.first.range; });
+
+  SemanticTokensPartialResult result;
+  int line = 0, column = 0;
+  for (auto &entry : scratch) {
+    lsRange &r = entry.first.range;
+    CclsSemanticHighlightSymbol &symbol = *entry.second;
+    if (r.start.line != line)
+      column = 0;
+    result.data.push_back(r.start.line - line);
+    line = r.start.line;
+    result.data.push_back(r.start.character - column);
+    column = r.start.character;
+    result.data.push_back(r.end.character - r.start.character);
+
+    int tokenType = (int)symbol.kind, modifier = 0;
+    if (tokenType == (int)SymbolKind::StaticMethod) {
+      tokenType = (int)SymbolKind::Method;
+      modifier |= 1 << (int)TokenModifier::Static;
+    } else if (tokenType >= (int)SymbolKind::FirstExtension) {
+      tokenType += (int)SymbolKind::FirstNonStandard - (int)SymbolKind::FirstExtension;
+    }
+
+    // Set modifiers.
+    if (entry.first.role & Role::Declaration)
+      modifier |= 1 << (int)TokenModifier::Declaration;
+    if (entry.first.role & Role::Definition)
+      modifier |= 1 << (int)TokenModifier::Definition;
+    if (entry.first.role & Role::Read)
+      modifier |= 1 << (int)TokenModifier::Read;
+    if (entry.first.role & Role::Write)
+      modifier |= 1 << (int)TokenModifier::Write;
+    if (symbol.storage == SC_Static)
+      modifier |= 1 << (int)TokenModifier::Static;
+
+    if (llvm::is_contained({SymbolKind::Constructor, SymbolKind::Field, SymbolKind::Method, SymbolKind::StaticMethod},
+                           symbol.kind))
+      modifier |= 1 << (int)TokenModifier::ClassScope;
+    else if (llvm::is_contained({SymbolKind::File, SymbolKind::Namespace}, symbol.parentKind))
+      modifier |= 1 << (int)TokenModifier::NamespaceScope;
+    else if (llvm::is_contained(
+                 {SymbolKind::Constructor, SymbolKind::Function, SymbolKind::Method, SymbolKind::StaticMethod},
+                 symbol.parentKind))
+      modifier |= 1 << (int)TokenModifier::FunctionScope;
+
+    // Rainbow semantic tokens
+    static_assert((int)TokenModifier::Id0 + 20 < 31);
+    if (int rainbow = g_config->highlight.rainbow)
+      modifier |= 1 << ((int)TokenModifier::Id0 + symbol.id % std::min(rainbow, 20));
+
+    result.data.push_back(tokenType);
+    result.data.push_back(modifier);
+  }
+
+  reply(result);
+}
+
 } // namespace ccls
