@@ -41,15 +41,12 @@ using namespace llvm;
 namespace ccls {
 std::pair<LanguageId, bool> lookupExtension(std::string_view filename) {
   using namespace clang::driver;
-  auto i = types::lookupTypeForExtension(
-      sys::path::extension({filename.data(), filename.size()}).substr(1));
-  bool header = i == types::TY_CHeader || i == types::TY_CXXHeader ||
-                i == types::TY_ObjCXXHeader;
+  auto i = types::lookupTypeForExtension(sys::path::extension({filename.data(), filename.size()}).substr(1));
+  bool header = i == types::TY_CHeader || i == types::TY_CXXHeader || i == types::TY_ObjCXXHeader;
   bool objc = types::isObjC(i);
   LanguageId ret;
   if (types::isCXX(i))
-    ret = types::isCuda(i) ? LanguageId::Cuda
-                           : objc ? LanguageId::ObjCpp : LanguageId::Cpp;
+    ret = types::isCuda(i) ? LanguageId::Cuda : objc ? LanguageId::ObjCpp : LanguageId::Cpp;
   else if (objc)
     ret = LanguageId::ObjC;
   else if (i == types::TY_C || i == types::TY_CHeader)
@@ -94,15 +91,12 @@ struct ProjectProcessor {
       i++;
       return true;
     }
-    return exclude_args.count(arg) ||
-           any_of(exclude_globs,
-                  [&](const GlobPattern &glob) { return glob.match(arg); });
+    return exclude_args.count(arg) || any_of(exclude_globs, [&](const GlobPattern &glob) { return glob.match(arg); });
   }
 
   // Expand %c %cpp ... in .ccls
   void process(Project::Entry &entry) {
-    std::vector<const char *> args(entry.args.begin(),
-                                   entry.args.begin() + entry.compdb_size);
+    std::vector<const char *> args(entry.args.begin(), entry.args.begin() + entry.compdb_size);
     auto [lang, header] = lookupExtension(entry.filename);
     for (int i = entry.compdb_size; i < entry.args.size(); i++) {
       const char *arg = entry.args[i];
@@ -134,93 +128,10 @@ struct ProjectProcessor {
       }
     }
     entry.args = args;
-    getSearchDirs(entry);
-  }
-
-  void getSearchDirs(Project::Entry &entry) {
-#if LLVM_VERSION_MAJOR < 8
-    const std::string base_name = sys::path::filename(entry.filename);
-    size_t hash = std::hash<std::string>{}(entry.directory);
-    bool OPT_o = false;
-    for (auto &arg : entry.args) {
-      bool last_o = OPT_o;
-      OPT_o = false;
-      if (arg[0] == '-') {
-        OPT_o = arg[1] == 'o' && arg[2] == '\0';
-        if (OPT_o || arg[1] == 'D' || arg[1] == 'W')
-          continue;
-      } else if (last_o) {
-        continue;
-      } else if (sys::path::filename(arg) == base_name) {
-        LanguageId lang = lookupExtension(arg).first;
-        if (lang != LanguageId::Unknown) {
-          hash_combine(hash, (size_t)lang);
-          continue;
-        }
-      }
-      hash_combine(hash, std::hash<std::string_view>{}(arg));
-    }
-    if (!command_set.insert(hash).second)
-      return;
-    auto args = entry.args;
-    args.push_back("-fsyntax-only");
-    for (const std::string &arg : g_config->clang.extraArgs)
-      args.push_back(intern(arg));
-    args.push_back(intern("-working-directory=" + entry.directory));
-    args.push_back(intern("-resource-dir=" + g_config->clang.resourceDir));
-
-    // a weird C++ deduction guide heap-use-after-free causes libclang to crash.
-    IgnoringDiagConsumer DiagC;
-    IntrusiveRefCntPtr<DiagnosticOptions> DiagOpts(new DiagnosticOptions());
-    DiagnosticsEngine Diags(
-        IntrusiveRefCntPtr<DiagnosticIDs>(new DiagnosticIDs()), &*DiagOpts,
-        &DiagC, false);
-
-    driver::Driver Driver(args[0], llvm::sys::getDefaultTargetTriple(), Diags);
-    auto TargetAndMode =
-        driver::ToolChain::getTargetAndModeFromProgramName(args[0]);
-    if (!TargetAndMode.TargetPrefix.empty()) {
-      const char *arr[] = {"-target", TargetAndMode.TargetPrefix.c_str()};
-      args.insert(args.begin() + 1, std::begin(arr), std::end(arr));
-      Driver.setTargetAndMode(TargetAndMode);
-    }
-    Driver.setCheckInputsExist(false);
-
-    std::unique_ptr<driver::Compilation> C(Driver.BuildCompilation(args));
-    const driver::JobList &Jobs = C->getJobs();
-    if (Jobs.size() != 1)
-      return;
-    const auto &CCArgs = Jobs.begin()->getArguments();
-
-    auto CI = std::make_unique<CompilerInvocation>();
-    CompilerInvocation::CreateFromArgs(*CI, CCArgs.data(),
-                                       CCArgs.data() + CCArgs.size(), Diags);
-    CI->getFrontendOpts().DisableFree = false;
-    CI->getCodeGenOpts().DisableFree = false;
-
-    HeaderSearchOptions &HeaderOpts = CI->getHeaderSearchOpts();
-    for (auto &E : HeaderOpts.UserEntries) {
-      std::string path =
-          normalizePath(resolveIfRelative(entry.directory, E.Path));
-      ensureEndsInSlash(path);
-      switch (E.Group) {
-      default:
-        folder.search_dir2kind[path] |= 2;
-        break;
-      case frontend::Quoted:
-        folder.search_dir2kind[path] |= 1;
-        break;
-      case frontend::Angled:
-        folder.search_dir2kind[path] |= 3;
-        break;
-      }
-    }
-#endif
   }
 };
 
-std::vector<const char *>
-readCompilerArgumentsFromFile(const std::string &path) {
+std::vector<const char *> readCompilerArgumentsFromFile(const std::string &path) {
   auto mbOrErr = MemoryBuffer::getFile(path);
   if (!mbOrErr)
     return {};
@@ -245,8 +156,7 @@ std::vector<const char *> getFallback(const std::string &path) {
   return argv;
 }
 
-void loadDirectoryListing(ProjectProcessor &proc, const std::string &root,
-                          const StringSet<> &seen) {
+void loadDirectoryListing(ProjectProcessor &proc, const std::string &root, const StringSet<> &seen) {
   Project::Folder &folder = proc.folder;
   std::vector<std::string> files;
 
@@ -257,8 +167,7 @@ void loadDirectoryListing(ProjectProcessor &proc, const std::string &root,
         return it->second;
       std::string normalized = normalizePath(cur);
       // Break if outside of the project root.
-      if (normalized.size() <= root.size() ||
-          normalized.compare(0, root.size(), root) != 0)
+      if (normalized.size() <= root.size() || normalized.compare(0, root.size(), root) != 0)
         break;
     }
     return folder.dot_ccls[root];
@@ -271,10 +180,8 @@ void loadDirectoryListing(ProjectProcessor &proc, const std::string &root,
                        if (!seen.count(path))
                          files.push_back(path);
                      } else if (sys::path::filename(path) == ".ccls") {
-                       std::vector<const char *> args =
-                           readCompilerArgumentsFromFile(path);
-                       folder.dot_ccls.emplace(
-                           sys::path::parent_path(path).str() + '/', args);
+                       std::vector<const char *> args = readCompilerArgumentsFromFile(path);
+                       folder.dot_ccls.emplace(sys::path::parent_path(path).str() + '/', args);
                        std::string l;
                        for (size_t i = 0; i < args.size(); i++) {
                          if (i)
@@ -399,14 +306,12 @@ void Project::loadDirectory(const std::string &root, Project::Folder &folder) {
         redir{StringRef(stdinPath), StringRef(path), StringRef()};
     std::vector<StringRef> args{g_config->compilationDatabaseCommand, root};
     if (sys::ExecuteAndWait(args[0], args, {}, redir, 0, 0, &err_msg) < 0) {
-      LOG_S(ERROR) << "failed to execute " << args[0].str() << " "
-                   << args[1].str() << ": " << err_msg;
+      LOG_S(ERROR) << "failed to execute " << args[0].str() << " " << args[1].str() << ": " << err_msg;
       return;
     }
   }
 
-  std::unique_ptr<tooling::CompilationDatabase> cdb =
-      tooling::CompilationDatabase::loadFromDirectory(cdbDir, err_msg);
+  std::unique_ptr<tooling::CompilationDatabase> cdb = tooling::CompilationDatabase::loadFromDirectory(cdbDir, err_msg);
   if (!g_config->compilationDatabaseCommand.empty()) {
 #ifdef _WIN32
     DeleteFileA(stdinPath.c_str());
@@ -439,8 +344,7 @@ void Project::loadDirectory(const std::string &root, Project::Folder &folder) {
       normalizeFolder(entry.directory);
       entry.directory.pop_back();
       doPathMapping(entry.directory);
-      entry.filename =
-          realPath(resolveIfRelative(entry.directory, cmd.Filename));
+      entry.filename = realPath(resolveIfRelative(entry.directory, cmd.Filename));
       normalizeFolder(entry.filename);
       doPathMapping(entry.filename);
 
@@ -452,7 +356,6 @@ void Project::loadDirectory(const std::string &root, Project::Folder &folder) {
           entry.args.push_back(intern(args[i]));
       }
       entry.compdb_size = entry.args.size();
-      proc.getSearchDirs(entry);
       if (seen.insert(entry.filename).second)
         folder.entries.push_back(entry);
     }
@@ -483,8 +386,7 @@ void Project::load(const std::string &root) {
   }
 }
 
-Project::Entry Project::findEntry(const std::string &path, bool can_redirect,
-                                  bool must_exist) {
+Project::Entry Project::findEntry(const std::string &path, bool can_redirect, bool must_exist) {
   std::string best_dot_ccls_root;
   Project::Folder *best_dot_ccls_folder = nullptr;
   std::string best_dot_ccls_dir;
@@ -501,8 +403,7 @@ Project::Entry Project::findEntry(const std::string &path, bool can_redirect,
     if (StringRef(path).startswith(root)) {
       // Find the best-fit .ccls
       for (auto &[dir, args] : folder.dot_ccls)
-        if (StringRef(path).startswith(dir) &&
-            dir.length() > best_dot_ccls_dir.length()) {
+        if (StringRef(path).startswith(dir) && dir.length() > best_dot_ccls_dir.length()) {
           best_dot_ccls_root = root;
           best_dot_ccls_folder = &folder;
           best_dot_ccls_dir = dir;
@@ -525,8 +426,7 @@ Project::Entry Project::findEntry(const std::string &path, bool can_redirect,
     }
 
   bool append = false;
-  if (best_dot_ccls_args && !(append = appendToCDB(*best_dot_ccls_args)) &&
-      !exact_match) {
+  if (best_dot_ccls_args && !(append = appendToCDB(*best_dot_ccls_args)) && !exact_match) {
     // If the first line is not %compile_commands.json, override the compdb
     // match if it is not an exact match.
     ret.root = ret.directory = best_dot_ccls_root;
@@ -582,8 +482,7 @@ Project::Entry Project::findEntry(const std::string &path, bool can_redirect,
   }
 
   if (best_dot_ccls_args && append && best_dot_ccls_args->size())
-    ret.args.insert(ret.args.end(), best_dot_ccls_args->begin() + 1,
-                    best_dot_ccls_args->end());
+    ret.args.insert(ret.args.end(), best_dot_ccls_args->begin() + 1, best_dot_ccls_args->end());
   if (best_compdb_folder)
     ProjectProcessor(*best_compdb_folder).process(ret);
   else if (best_dot_ccls_folder)
@@ -591,15 +490,22 @@ Project::Entry Project::findEntry(const std::string &path, bool can_redirect,
   for (const std::string &arg : g_config->clang.extraArgs)
     ret.args.push_back(intern(arg));
   ret.args.push_back(intern("-working-directory=" + ret.directory));
+  if (g_config->clang.prependArgs.size()) {
+    std::vector<const char *> args;
+    for (StringRef arg : g_config->clang.prependArgs)
+      args.push_back(intern(arg));
+    ret.args.insert(ret.args.begin() + 1, args.begin(), args.end());
+  }
   return ret;
 }
 
 void Project::index(WorkingFiles *wfiles, const RequestId &id) {
   auto &gi = g_config->index;
-  GroupMatch match(gi.whitelist, gi.blacklist),
-      match_i(gi.initialWhitelist, gi.initialBlacklist);
-  std::vector<const char *> args, extra_args;
-  for (const std::string &arg : g_config->clang.extraArgs)
+  GroupMatch match(gi.whitelist, gi.blacklist), match_i(gi.initialWhitelist, gi.initialBlacklist);
+  std::vector<const char *> args, prepend_args, extra_args;
+  for (StringRef arg : g_config->clang.prependArgs)
+    prepend_args.push_back(intern(arg));
+  for (StringRef arg : g_config->clang.extraArgs)
     extra_args.push_back(intern(arg));
   {
     std::lock_guard lock(mtx);
@@ -607,19 +513,15 @@ void Project::index(WorkingFiles *wfiles, const RequestId &id) {
       int i = 0;
       for (const Project::Entry &entry : folder.entries) {
         std::string reason;
-        if (match.matches(entry.filename, &reason) &&
-            match_i.matches(entry.filename, &reason)) {
+        if (match.matches(entry.filename, &reason) && match_i.matches(entry.filename, &reason)) {
           bool interactive = wfiles->getFile(entry.filename) != nullptr;
           args = entry.args;
           args.insert(args.end(), extra_args.begin(), extra_args.end());
           args.push_back(intern("-working-directory=" + entry.directory));
-          pipeline::index(entry.filename, args,
-                          interactive ? IndexMode::Normal
-                                      : IndexMode::Background,
-                          false, id);
+          args.insert(args.begin() + 1, prepend_args.begin(), prepend_args.end());
+          pipeline::index(entry.filename, args, interactive ? IndexMode::Normal : IndexMode::Background, false, id);
         } else {
-          LOG_V(1) << "[" << i << "/" << folder.entries.size()
-                   << "]: " << reason << "; skip " << entry.filename;
+          LOG_V(1) << "[" << i << "/" << folder.entries.size() << "]: " << reason << "; skip " << entry.filename;
         }
         i++;
       }
@@ -636,8 +538,10 @@ void Project::indexRelated(const std::string &path) {
   auto &gi = g_config->index;
   GroupMatch match(gi.whitelist, gi.blacklist);
   StringRef stem = sys::path::stem(path);
-  std::vector<const char *> args, extra_args;
-  for (const std::string &arg : g_config->clang.extraArgs)
+  std::vector<const char *> args, prepend_args, extra_args;
+  for (StringRef arg : g_config->clang.prependArgs)
+    prepend_args.push_back(intern(arg));
+  for (StringRef arg : g_config->clang.extraArgs)
     extra_args.push_back(intern(arg));
   std::lock_guard lock(mtx);
   for (auto &[root, folder] : root2folder)
@@ -647,8 +551,8 @@ void Project::indexRelated(const std::string &path) {
         args = entry.args;
         args.insert(args.end(), extra_args.begin(), extra_args.end());
         args.push_back(intern("-working-directory=" + entry.directory));
-        if (sys::path::stem(entry.filename) == stem && entry.filename != path &&
-            match.matches(entry.filename, &reason))
+        args.insert(args.begin() + 1, prepend_args.begin(), prepend_args.end());
+        if (sys::path::stem(entry.filename) == stem && entry.filename != path && match.matches(entry.filename, &reason))
           pipeline::index(entry.filename, args, IndexMode::Background, true);
       }
       break;
